import click
from click_option_group import (
    optgroup, 
    MutuallyExclusiveOptionGroup,
    RequiredMutuallyExclusiveOptionGroup,
)
from rich import print
from ciphit.basemods.Ciphers import aes_cbc

def get_singleline(multi: list) -> repr:
    assert isinstance(multi,(list,str))
    return repr('\n'.join([i.strip('\n') for i in multi]))

def get_multiline(single: str) -> str:
    return '\n'.join(single.strip("'").split("\\n"))

def print_help():
    ctx = click.get_current_context()
    click.echo(ctx.get_help())

@click.command()
@optgroup.group(
        "Encode/Decode", 
        cls=RequiredMutuallyExclusiveOptionGroup,
)
@optgroup.option(
        "-e", "--encode", is_flag=True, flag_value=True,
)
@optgroup.option(
        "-d", "--decode", is_flag=True, flag_value=True,
)
@optgroup.option(
    "--edit", 
    help="To edit Encrypted/Encoded files created by ciphit.", 
    is_flag=True, 
    flag_value=True,
)
@click.option(
        "-k","--key", 
        help="The key with which text is Encoded/Decoded.",
        default=False,
)
@optgroup.group(
        "Text/File", 
        cls=MutuallyExclusiveOptionGroup,
)
@optgroup.option(
    "-t", "--text",
    help="The text you want to Encode/Decode.", 
    default=False, type=str,
)
@optgroup.option(
    "-f", "--file", 
    type=click.File('r+'), 
    default=None,
)
def main(**kwargs):

    isFile = not kwargs['file'] is None
    
    if kwargs['edit']:
        if isFile or kwargs['key']:
            pass
        else:
            raise click.UsageError(
                "'--edit' flag is only allowed with '-f' / '--file'.\nhelp:\t'--edit'\n\t'-f' / '--file' [required]\n\t'-t' / '--text' [optional]"
            )

    if not kwargs['key']:
        kwargs['key'] = click.prompt('Key', 
                        hide_input=True, 
                        confirmation_prompt=True)

    if not kwargs['text'] and (not kwargs['edit'] and not isFile):
        print("[bold blue]Opening editor[/bold blue]")
        click.pause()
        kwargs['text']=click.edit(text='')
        if kwargs['text'] in (None,''):
            print("[bold red]error:[/bold red] Text is empty.")
            exit(1)
        kwargs['text']=kwargs['text'].strip('\n')

    if kwargs['edit']:
        deciphered = get_multiline(aes_cbc.PassDecode(kwargs['file'].read().strip(), kwargs['key']))
        kwargs['text']=click.edit(text=deciphered)
        if kwargs['text'] is None: exit(1)
        ciphered = aes_cbc.PassEncode(get_singleline(kwargs['text'].split('\n')), kwargs['key'])
        kwargs['file'].truncate(0)
        kwargs['file'].seek(0)
        kwargs['file'].write(ciphered)
    elif not kwargs['edit']:
        if not isFile:
            if kwargs['encode']:
                _ = aes_cbc.PassEncode(kwargs['text'],kwargs['key'])
                print(f"Final result: [bold yellow]{_}[/bold yellow]")
            else:#kwargs['decode']
                _ = aes_cbc.PassDecode(kwargs['text'],kwargs['key'])
                print(f"Final result: [bold green]{_}[/bold green]")
        elif isFile:
            if kwargs['encode']:
                _ = aes_cbc.PassEncode(get_singleline(kwargs['file'].readlines()), kwargs['key'])
                msg="[bold green]File is now enncrypted.[/bold green]"
            else:#kwargs['decode']
                _ = get_multiline(aes_cbc.PassDecode(kwargs['file'].read().strip(), kwargs['key']))
                msg="[bold green]File is now decrypted.[/bold green]"
            kwargs['file'].truncate(0)
            kwargs['file'].seek(0)
            kwargs['file'].write(_)
            print(msg)
        else:
            #this condition never occurs, for now.
            print_help()
            exit(1)

if __name__ == '__main__':
    main()



import secrets
from base64 import urlsafe_b64encode as b64e, urlsafe_b64decode as b64d

from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

backend = default_backend()
iterations = 100_000

def Encode(message: bytes, key=Fernet.generate_key()) -> bytes:
    return Fernet(key).encrypt(message.encode()).decode(), key

def Decode(token: bytes, key: bytes) -> bytes:
    return Fernet(key).decrypt(token.encode()).decode()

def _derive_key(password: bytes, salt: bytes, iterations: int = iterations) -> bytes:
    """Derive a secret key from a given password and salt"""
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(), length=32, salt=salt,
        iterations=iterations, backend=backend)
    return b64e(kdf.derive(password))

def PassEncode(message: bytes, password: str, iterations: int = iterations) -> bytes:
    salt = secrets.token_bytes(16)
    key = _derive_key(password.encode(), salt, iterations)
    return b64e(
        b'%b%b%b' % (
            salt,
            iterations.to_bytes(4, 'big'),
            b64d(Fernet(key).encrypt(message.encode())),
        )
    ).decode()

def PassDecode(token: bytes, password: str) -> bytes:
    decoded = b64d(token)
    salt, iter, token = decoded[:16], decoded[16:20], b64e(decoded[20:])
    iterations = int.from_bytes(iter, 'big')
    key = _derive_key(password.encode(), salt, iterations)
    return Fernet(key).decrypt(token).decode()


import setuptools


with open('README.md', 'r') as readme_fp:
    long_description = readme_fp.read()

with open('requirements.txt', 'r') as req_fp:
    required_libs = req_fp.readlines()


setuptools.setup(
    name='ciphit',
    scripts=['ciphit/__main__.py'],
    description='ciphit is a cryptography CLI-tool',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='0.5.0',
    author='Sagar Kumar',
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=setuptools.find_packages(exclude=['install.sh']),
    install_requires=required_libs,
    url='https://github.com/sgrkmr/ciphit',
    keywords='cli commandline user-interface ui cryptography',
    python_requires='>=3.7',
    entry_points={
    'console_scripts': [
        'ciphit = ciphit.__main__:main',
        ],
    },
)
